@@ -33,7 +33,7 @@ from ansible_compat.ports import cache, cached_property
 from packaging.version import Version
 
 from molecule import api, interpolation, platforms, scenario, state, util
-from molecule.app import app
+from molecule.app import get_app
 from molecule.data import __file__ as data_module
 from molecule.dependency import ansible_galaxy, shell
 from molecule.model import schema_v3
@@ -43,6 +43,13 @@ from molecule.util import boolean
 
 if TYPE_CHECKING:
     from collections.abc import MutableMapping
+    from typing import Literal
+
+    from molecule.dependency.base import Base as Dependency
+    from molecule.driver.base import Driver
+    from molecule.state import State
+    from molecule.types import CollectionData, CommandArgs, ConfigData, MoleculeArgs
+    from molecule.verifier.base import Verifier
 
 
 LOG = logging.getLogger(__name__)
@@ -58,25 +65,19 @@ MOLECULE_EMBEDDED_DATA_DIR = os.path.dirname(data_module)  # noqa: PTH120
 
 @cache
 def ansible_version() -> Version:
-    """Retrieve Ansible version."""
+    """Retrieve Ansible version.
+
+    Returns:
+        Molecule version information.
+    """
     warnings.warn(  # noqa: B028
         "molecule.config.ansible_version is deprecated, will be removed in the future.",
         category=DeprecationWarning,
     )
-    return app.runtime.version
+    return get_app(Path()).runtime.version
 
 
-# https://stackoverflow.com/questions/16017397/injecting-function-call-after-init-with-decorator
-class NewInitCaller(type):
-    """NewInitCaller."""
-
-    def __call__(cls, *args, **kwargs):  # type: ignore[no-untyped-def]  # noqa: ANN002, ANN003, ANN204, D102
-        obj = type.__call__(cls, *args, **kwargs)
-        obj.after_init()
-        return obj
-
-
-class Config(metaclass=NewInitCaller):
+class Config:
     """Config Class.
 
     Molecule searches the current directory for ``molecule.yml`` files by
@@ -94,12 +95,12 @@ class Config(metaclass=NewInitCaller):
 
     # pylint: disable=too-many-instance-attributes
     # Config objects should be allowed to have any number of attributes
-    def __init__(  # type: ignore[no-untyped-def]
+    def __init__(
         self,
         molecule_file: str,  # pylint: disable=redefined-outer-name
-        args={},  # noqa: ANN001, B006
-        command_args={},  # noqa: ANN001, B006
-        ansible_args=(),  # noqa: ANN001
+        args: MoleculeArgs = {},  # noqa: B006
+        command_args: CommandArgs = {},  # noqa: B006
+        ansible_args: tuple[str, ...] = (),
     ) -> None:
         """Initialize a new config class and returns None.
 
@@ -114,73 +115,186 @@ class Config(metaclass=NewInitCaller):
         self.command_args = command_args
         self.ansible_args = ansible_args
         self.config = self._get_config()
-        self._action = None
+        self._action: str | None = None
         self._run_uuid = str(uuid4())
         self.project_directory = os.getenv(
             "MOLECULE_PROJECT_DIRECTORY",
             os.getcwd(),  # noqa: PTH109
         )
-        self.runtime = app.runtime
+        self.app = get_app(Path(self.project_directory))
+        self.runtime = self.app.runtime
         self.scenario_path = Path(molecule_file).parent
 
-    def after_init(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
-        self.config = self._reget_config()  # type: ignore[no-untyped-call]
+        # Former after_init() contents
+        self.config = self._reget_config()
         if self.molecule_file:
-            self._validate()  # type: ignore[no-untyped-call]
+            self._validate()
 
-    def write(self) -> None:  # noqa: D102
+    def write(self) -> None:
+        """Write config file to filesystem."""
         util.write_file(self.config_file, util.safe_dump(self.config))
 
     @property
-    def ansible_collections_path(self):  # type: ignore[no-untyped-def]  # noqa: ANN201
-        """Return collection path variable for current version of Ansible."""
+    def ansible_collections_path(
+        self,
+    ) -> str:
+        """Return collection path variable for current version of Ansible.
+
+        Returns:
+            The correct ansible collection path to use.
+        """
         # https://github.com/ansible/ansible/pull/70007
         if self.runtime.version >= Version("2.10.0.dev0"):
             return "ANSIBLE_COLLECTIONS_PATH"
         return "ANSIBLE_COLLECTIONS_PATHS"
 
     @property
-    def config_file(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
-        return os.path.join(self.scenario.ephemeral_directory, MOLECULE_FILE)  # noqa: PTH118
+    def config_file(self) -> str:
+        """Path to the config file.
+
+        Returns:
+            The path to the config file.
+        """
+        path = Path(self.scenario.ephemeral_directory) / MOLECULE_FILE
+        return str(path)
 
     @property
-    def is_parallel(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def is_parallel(self) -> bool:
+        """Is molecule in parallel mode.
+
+        Returns:
+            Whether molecule is in parallel mode.
+        """
         return self.command_args.get("parallel", False)
 
     @property
-    def platform_name(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def platform_name(self) -> str | None:
+        """Configured platform.
+
+        Returns:
+            The name of the platform plugin specified.
+        """
         return self.command_args.get("platform_name", None)
 
     @property
-    def debug(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def debug(self) -> bool:
+        """Is molecule in debug mode.
+
+        Returns:
+            Whether molecule is in debug mode.
+        """
         return self.args.get("debug", MOLECULE_DEBUG)
 
     @property
-    def env_file(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def env_file(self) -> str | None:
+        """Return path to env file.
+
+        Returns:
+            Path to configured env file.
+        """
         return util.abs_path(self.args.get("env_file"))
 
     @property
-    def subcommand(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def subcommand(self) -> str:
+        """Subcommand in use.
+
+        Returns:
+            The current subcommand being run.
+        """
         return self.command_args["subcommand"]
 
     @property
-    def action(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def action(self) -> str | None:
+        """Action value.
+
+        Returns:
+            The value of action.
+        """
         return self._action
 
     @action.setter
-    def action(self, value):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN202
+    def action(self, value: str) -> None:
+        """Action setter.
+
+        Args:
+            value: New value for action.
+        """
         self._action = value
 
     @property
-    def cache_directory(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def cache_directory(
+        self,
+    ) -> Literal["molecule", "molecule_parallel"]:
+        """Proper cache directory to use.
+
+        Returns:
+            Either "molecule" or "molecule_parallel" if in parallel mode.
+        """
         return "molecule_parallel" if self.is_parallel else "molecule"
 
     @property
-    def molecule_directory(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def collection_directory(self) -> Path | None:
+        """Location of collection containing the molecule files.
+
+        Returns:
+            Root of the collection containing the molecule files.
+        """
+        test_paths = [Path.cwd(), Path(self.project_directory)]
+
+        for path in test_paths:
+            if (path / "galaxy.yml").exists():
+                return path
+
+        # Last resort, try to find git root
+        show_toplevel = self.app.run_command("git rev-parse --show-toplevel")
+        if show_toplevel.returncode == 0:
+            path = Path(show_toplevel.stdout.strip())
+            if (path / "galaxy.yml").exists():
+                return path
+
+        return None
+
+    @property
+    def molecule_directory(self) -> str:
+        """Molecule directory for this project.
+
+        Returns:
+            The appropriate molecule directory for this project.
+        """
         return molecule_directory(self.project_directory)
 
     @cached_property
-    def dependency(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def collection(self) -> CollectionData | None:
+        """Collection metadata sourced from galaxy.yml.
+
+        Returns:
+            A dictionary of information about the collection molecule is running inside, if any.
+        """
+        collection_directory = self.collection_directory
+        if not collection_directory:
+            return None
+
+        galaxy_file = collection_directory / "galaxy.yml"
+        galaxy_data: CollectionData = util.safe_load_file(galaxy_file)
+
+        important_keys = {"name", "namespace"}
+        if missing_keys := important_keys.difference(galaxy_data.keys()):
+            LOG.warning(
+                "The detected galaxy.yml file (%s) is invalid, missing mandatory field %s",
+                galaxy_file,
+                util.oxford_comma(missing_keys),
+            )
+            return None  # pragma: no cover
+
+        return galaxy_data
+
+    @cached_property
+    def dependency(self) -> Dependency | None:
+        """Dependency manager in use.
+
+        Returns:
+            Instance of a molecule dependency plugin.
+        """
         dependency_name = self.config["dependency"]["name"]
         if dependency_name == "galaxy":
             return ansible_galaxy.AnsibleGalaxy(self)
@@ -189,14 +303,18 @@ class Config(metaclass=NewInitCaller):
         return None
 
     @cached_property
-    def driver(self):  # type: ignore[no-untyped-def] # noqa: ANN201
-        """Return driver name."""
-        driver_name = self._get_driver_name()  # type: ignore[no-untyped-call]
+    def driver(self) -> Driver:
+        """Return driver.
+
+        Returns:
+            The driver for this scenario.
+        """
+        driver_name = self._get_driver_name()
         driver = None
 
         api_drivers = api.drivers(config=self)
         if driver_name not in api_drivers:
-            msg = f"Failed to find driver {driver_name}. Please ensure that the driver is correctly installed."  # noqa: E501
+            msg = f"Failed to find driver {driver_name}. Please ensure that the driver is correctly installed."
             util.sysexit_with_message(msg)
 
         driver = api_drivers[driver_name]
@@ -205,27 +323,37 @@ class Config(metaclass=NewInitCaller):
         return driver
 
     @property
-    def env(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def env(self) -> dict[str, str]:
+        """Environment variables.
+
+        Returns:
+            Total set of computed environment variables.
+        """
         return {
             "MOLECULE_DEBUG": str(self.debug),
             "MOLECULE_FILE": self.config_file,
             "MOLECULE_ENV_FILE": str(self.env_file),
             "MOLECULE_STATE_FILE": self.state.state_file,
-            "MOLECULE_INVENTORY_FILE": self.provisioner.inventory_file,
+            "MOLECULE_INVENTORY_FILE": self.provisioner.inventory_file,  # type: ignore[union-attr]
             "MOLECULE_EPHEMERAL_DIRECTORY": self.scenario.ephemeral_directory,
             "MOLECULE_SCENARIO_DIRECTORY": self.scenario.directory,
             "MOLECULE_PROJECT_DIRECTORY": self.project_directory,
             "MOLECULE_INSTANCE_CONFIG": self.driver.instance_config,
-            "MOLECULE_DEPENDENCY_NAME": self.dependency.name,
+            "MOLECULE_DEPENDENCY_NAME": self.dependency.name,  # type: ignore[union-attr]
             "MOLECULE_DRIVER_NAME": self.driver.name,
-            "MOLECULE_PROVISIONER_NAME": self.provisioner.name,
+            "MOLECULE_PROVISIONER_NAME": self.provisioner.name,  # type: ignore[union-attr]
             "MOLECULE_SCENARIO_NAME": self.scenario.name,
             "MOLECULE_VERIFIER_NAME": self.verifier.name,
             "MOLECULE_VERIFIER_TEST_DIRECTORY": self.verifier.directory,
         }
 
     @cached_property
-    def platforms(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def platforms(self) -> platforms.Platforms:
+        """Platforms for this run.
+
+        Returns:
+            A molecule Platforms instance.
+        """
         return platforms.Platforms(
             self,
             parallelize_platforms=self.is_parallel,
@@ -233,18 +361,33 @@ class Config(metaclass=NewInitCaller):
         )
 
     @cached_property
-    def provisioner(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def provisioner(self) -> ansible.Ansible | None:
+        """Provisioner for this run.
+
+        Returns:
+            An instance of the Ansible provisioner.
+        """
         provisioner_name = self.config["provisioner"]["name"]
         if provisioner_name == "ansible":
             return ansible.Ansible(self)
         return None
 
     @cached_property
-    def scenario(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def scenario(self) -> scenario.Scenario:
+        """Scenario for this run.
+
+        Returns:
+            A molecule Scenario instance.
+        """
         return scenario.Scenario(self)
 
     @cached_property
-    def state(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def state(self) -> State:
+        """Molecule state object.
+
+        Returns:
+            A molecule State instance.
+        """
         myState = state.State(self)  # noqa: N806
         # look at state file for molecule.yml date modified and warn if they do not match
         if self.molecule_file and os.path.isfile(self.molecule_file):  # noqa: PTH113
@@ -253,8 +396,8 @@ class Config(metaclass=NewInitCaller):
                 myState.change_state("molecule_yml_date_modified", modTime)
             elif myState.molecule_yml_date_modified != modTime:
                 LOG.warning(
-                    "The scenario config file ('%s') has been modified since the scenario was created. "  # noqa: E501
-                    "If recent changes are important, reset the scenario with 'molecule destroy' to clean up created items or "  # noqa: E501
+                    "The scenario config file ('%s') has been modified since the scenario was created. "
+                    "If recent changes are important, reset the scenario with 'molecule destroy' to clean up created items or "
                     "'molecule reset' to clear current configuration.",
                     self.molecule_file,
                 )
@@ -262,10 +405,22 @@ class Config(metaclass=NewInitCaller):
         return state.State(self)
 
     @cached_property
-    def verifier(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
-        return api.verifiers(self).get(self.config["verifier"]["name"], None)  # type: ignore[no-untyped-call]
+    def verifier(self) -> Verifier:
+        """Retrieve current verifier.
 
-    def _get_driver_name(self):  # type: ignore[no-untyped-def] # noqa: ANN202
+        Raises:
+            RuntimeError: If is not able to find the driver.
+
+        Returns:
+            Instance of Verifier driver.
+        """
+        name = self.config["verifier"]["name"]
+        if name not in api.verifiers(self):
+            msg = f"Unable to find '{name}' verifier driver."
+            raise RuntimeError(msg)
+        return api.verifiers(self)[name]
+
+    def _get_driver_name(self) -> str:
         # the state file contains the driver from the last run
         driver_from_state_file = self.state.driver
         # the user may supply a driver on the command line
@@ -300,13 +455,13 @@ class Config(metaclass=NewInitCaller):
             msg = (
                 f"Driver '{driver_name}' is currently in use but the scenario config "
                 f"has changed and now defines '{driver_from_scenario}'. "
-                "To change drivers, run 'molecule destroy' for converged scenarios or 'molecule reset' otherwise."  # noqa: E501
+                "To change drivers, run 'molecule destroy' for converged scenarios or 'molecule reset' otherwise."
             )
             LOG.warning(msg)
 
         return driver_name
 
-    def _get_config(self) -> MutableMapping:  # type: ignore[type-arg]
+    def _get_config(self) -> ConfigData:
         """Perform a prioritized recursive merge of config files.
 
         Returns a new dict.  Prior to merging the config files are interpolated with
@@ -317,7 +472,7 @@ class Config(metaclass=NewInitCaller):
         """
         return self._combine(keep_string=MOLECULE_KEEP_STRING)
 
-    def _reget_config(self):  # type: ignore[no-untyped-def]  # noqa: ANN202
+    def _reget_config(self) -> ConfigData:
         """Perform the same prioritized recursive merge from `get_config`.
 
         Interpolates the ``keep_string`` left behind in the original
@@ -331,7 +486,11 @@ class Config(metaclass=NewInitCaller):
 
         return self._combine(env=env)
 
-    def _combine(self, env=os.environ, keep_string=None) -> MutableMapping:  # type: ignore[no-untyped-def, type-arg]  # noqa: ANN001
+    def _combine(
+        self,
+        env: MutableMapping[str, str] = os.environ,
+        keep_string: str | None = None,
+    ) -> ConfigData:
         """Perform a prioritized recursive merge of config files.
 
         Returns a new dict.  Prior to merging the config files are interpolated with
@@ -342,6 +501,10 @@ class Config(metaclass=NewInitCaller):
         3. Loads the scenario's ``molecule file`` and merges on top of previous
            merge.
 
+        Args:
+            env: The current set of environment variables to consider.
+            keep_string: String to avoid templating.
+
         Returns:
             dict: The merged config.
         """
@@ -367,7 +530,12 @@ class Config(metaclass=NewInitCaller):
 
         return defaults
 
-    def _interpolate(self, stream: str, env: MutableMapping, keep_string: str) -> str:  # type: ignore[type-arg]
+    def _interpolate(
+        self,
+        stream: str,
+        env: MutableMapping[str, str],
+        keep_string: str | None,
+    ) -> str:
         env = set_env_from_file(env, self.env_file)
 
         i = interpolation.Interpolator(interpolation.TemplateWithDefaults, env)
@@ -379,7 +547,7 @@ class Config(metaclass=NewInitCaller):
             util.sysexit_with_message(msg)
         return ""
 
-    def _get_defaults(self) -> MutableMapping:  # type: ignore[type-arg]
+    def _get_defaults(self) -> ConfigData:
         if not self.molecule_file:
             scenario_name = "default"
         else:
@@ -472,28 +640,56 @@ class Config(metaclass=NewInitCaller):
             },
         }
 
-    def _validate(self):  # type: ignore[no-untyped-def]  # noqa: ANN202
+    def _validate(self) -> None:
+        """Validate molecule file."""
         msg = f"Validating schema {self.molecule_file}."
         LOG.debug(msg)
 
-        errors = schema_v3.validate(self.config)  # type: ignore[no-untyped-call]
+        errors = schema_v3.validate(self.config)
         if errors:
             msg = f"Failed to validate {self.molecule_file}\n\n{errors}"
             util.sysexit_with_message(msg)
 
 
-def molecule_directory(path: str) -> str:
-    """Return directory of the current scenario."""
-    return os.path.join(path, MOLECULE_DIRECTORY)  # noqa: PTH118
+def molecule_directory(path: str | Path) -> str:
+    """Return directory of the current scenario.
+
+    Args:
+        path: Base molecule directory.
+
+    Returns:
+        The current scenario's directory.
+    """
+    if isinstance(path, str):
+        path = Path(path)
+    return str(path / MOLECULE_DIRECTORY)
 
 
 def molecule_file(path: str) -> str:
-    """Return file path of current scenario."""
+    """Return file path of current scenario.
+
+    Args:
+        path: Base molecule directory.
+
+    Returns:
+        The path to the molecule file.
+    """
     return os.path.join(path, MOLECULE_FILE)  # noqa: PTH118
 
 
-def set_env_from_file(env: MutableMapping[str, str], env_file: str) -> MutableMapping:  # type: ignore[type-arg]
-    """Load environment from file."""
+def set_env_from_file(
+    env: MutableMapping[str, str],
+    env_file: str | None,
+) -> MutableMapping[str, str]:
+    """Load environment from file.
+
+    Args:
+        env: Currently known environment variables.
+        env_file: File from which to load more environment variables.
+
+    Returns:
+        The combined set of environment variables.
+    """
     if env_file and os.path.exists(env_file):  # noqa: PTH110
         env = copy.copy(env)
         d = util.safe_load_file(env_file)
