@@ -1,3 +1,4 @@
+# pylint: disable=too-many-lines
 #  Copyright (c) 2015-2018 Cisco Systems, Inc.
 
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -27,7 +28,8 @@ import logging
 import os
 import shutil
 
-from typing import Any
+from pathlib import Path
+from typing import TYPE_CHECKING
 
 from ansible_compat.ports import cached_property
 
@@ -36,6 +38,14 @@ from molecule.api import drivers
 from molecule.provisioner import ansible_playbook, ansible_playbooks, base
 
 
+if TYPE_CHECKING:
+    from typing import Any
+
+    from molecule.types import Options
+
+    Vivify = collections.defaultdict[str, Any | "Vivify"]
+
+
 LOG = logging.getLogger(__name__)
 
 
@@ -413,17 +423,13 @@ class Ansible(base.Base):
     ```
     """
 
-    def __init__(self, config) -> None:  # type: ignore[no-untyped-def]  # pylint: disable=useless-parent-delegation  # noqa: ANN001
-        """Initialize a new ansible class and returns None.
-
-        Args:
-            config: An instance of a Molecule config.
-        """
-        super().__init__(config)
-
     @property
     def default_config_options(self) -> dict[str, Any]:
-        """Provide Default options to construct ansible.cfg and returns a dict."""
+        """Provide default options to construct ansible.cfg.
+
+        Returns:
+            Default config options.
+        """
         return {
             "defaults": {
                 "ansible_managed": "Ansible managed: Do NOT edit this file manually!",
@@ -441,20 +447,30 @@ class Ansible(base.Base):
         }
 
     @property
-    def default_options(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
-        d = {"skip-tags": "molecule-notest,notest"}
+    def default_options(self) -> dict[str, str | bool]:
+        """Provide default options.
+
+        Returns:
+            Default options.
+        """
+        d: dict[str, str | bool] = {"skip-tags": "molecule-notest,notest"}
 
         if self._config.action == "idempotence":
-            d["skip-tags"] += ",molecule-idempotence-notest"
+            d["skip-tags"] += ",molecule-idempotence-notest"  # type: ignore[assignment, operator]
 
         if self._config.debug:
-            d["vvv"] = True  # type: ignore[assignment]
-            d["diff"] = True  # type: ignore[assignment]
+            d["vvv"] = True
+            d["diff"] = True
 
         return d
 
     @property
-    def default_env(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def default_env(self) -> dict[str, str]:
+        """Provide default environment variables.
+
+        Returns:
+            Default set of environment variables.
+        """
         # Finds if the current project is part of an ansible_collections hierarchy
         collection_indicator = "ansible_collections"
         # isolating test environment by injects ephemeral scenario directory on
@@ -517,11 +533,11 @@ class Ansible(base.Base):
             )
 
         env = util.merge_dicts(
-            os.environ,
+            dict(os.environ),
             {
-                "ANSIBLE_CONFIG": self._config.provisioner.config_file,
-                "ANSIBLE_ROLES_PATH": ":".join(roles_path_list),  # type: ignore[arg-type]
-                self._config.ansible_collections_path: ":".join(collections_path_list),  # type: ignore[arg-type]
+                "ANSIBLE_CONFIG": self.config_file,
+                "ANSIBLE_ROLES_PATH": ":".join(roles_path_list),
+                self._config.ansible_collections_path: ":".join(collections_path_list),
                 "ANSIBLE_LIBRARY": ":".join(self._get_modules_directories()),
                 "ANSIBLE_FILTER_PLUGINS": ":".join(
                     self._get_filter_plugins_directories(),
@@ -533,35 +549,60 @@ class Ansible(base.Base):
         return env  # noqa: RET504
 
     @property
-    def name(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def name(self) -> str:
+        """Provisioner name.
+
+        Returns:
+            The provisioner name.
+        """
         return self._config.config["provisioner"]["name"]
 
     @property
-    def ansible_args(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def ansible_args(self) -> list[str]:
+        """Provisioner ansible args.
+
+        Returns:
+            The provisioner ansible_args.
+        """
         return self._config.config["provisioner"]["ansible_args"]
 
     @property
-    def config_options(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def config_options(self) -> dict[str, Any]:
+        """Provisioner config options.
+
+        Returns:
+            The provisioner config options.
+        """
         return util.merge_dicts(
             self.default_config_options,
             self._config.config["provisioner"]["config_options"],
         )
 
     @property
-    def options(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def options(self) -> Options:
+        """Appropriate options for provisioner.
+
+        Returns:
+            Dictionary of provisioner options.
+        """
         if self._config.action in ["create", "destroy"]:
             return self.default_options
 
-        o = self._config.config["provisioner"]["options"]
+        opts = self._config.config["provisioner"]["options"]
         # NOTE(retr0h): Remove verbose options added by the user while in
         # debug.
         if self._config.debug:
-            o = util.filter_verbose_permutation(o)  # type: ignore[no-untyped-call]
+            opts = util.filter_verbose_permutation(opts)
 
-        return util.merge_dicts(self.default_options, o)
+        return util.merge_dicts(self.default_options, opts)
 
     @property
-    def env(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def env(self) -> dict[str, str]:
+        """Full computed environment variables for provisioner.
+
+        Returns:
+            Complete set of collected environment variables.
+        """
         default_env = self.default_env
         env = self._config.config["provisioner"]["env"].copy()
         # ensure that all keys and values are strings
@@ -571,13 +612,13 @@ class Ansible(base.Base):
         filter_plugins_path = default_env["ANSIBLE_FILTER_PLUGINS"]
 
         try:
-            path = self._absolute_path_for(env, "ANSIBLE_LIBRARY")  # type: ignore[no-untyped-call]
+            path = self._absolute_path_for(env, "ANSIBLE_LIBRARY")
             library_path = f"{library_path}:{path}"
         except KeyError:
             pass
 
         try:
-            path = self._absolute_path_for(env, "ANSIBLE_FILTER_PLUGINS")  # type: ignore[no-untyped-call]
+            path = self._absolute_path_for(env, "ANSIBLE_FILTER_PLUGINS")
             filter_plugins_path = f"{filter_plugins_path}:{path}"
         except KeyError:
             pass
@@ -588,23 +629,43 @@ class Ansible(base.Base):
         return util.merge_dicts(default_env, env)
 
     @property
-    def hosts(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def hosts(self) -> dict[str, str]:
+        """Provisioner inventory hosts.
+
+        Returns:
+            Dictionary of host names.
+        """
         return self._config.config["provisioner"]["inventory"]["hosts"]
 
     @property
-    def host_vars(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def host_vars(self) -> dict[str, str]:
+        """Provisioner inventory host vars.
+
+        Returns:
+            Dictionary of host vars.
+        """
         return self._config.config["provisioner"]["inventory"]["host_vars"]
 
     @property
-    def group_vars(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def group_vars(self) -> dict[str, str]:
+        """Provisioner inventory group vars.
+
+        Returns:
+            Dictionary of group vars.
+        """
         return self._config.config["provisioner"]["inventory"]["group_vars"]
 
     @property
-    def links(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def links(self) -> dict[str, str]:
+        """Provisioner inventory links.
+
+        Returns:
+            Dictionary of links.
+        """
         return self._config.config["provisioner"]["inventory"]["links"]
 
     @property
-    def inventory(self):  # type: ignore[no-untyped-def]  # noqa: ANN201
+    def inventory(self) -> dict[str, str]:
         """Create an inventory structure and returns a dict.
 
         ``` yaml
@@ -627,15 +688,15 @@ class Ansible(base.Base):
                   ansible_connection: docker
         ```
         """
-        dd = self._vivify()  # type: ignore[no-untyped-call]
+        dd = self._vivify()
         for platform in self._config.platforms.instances:
             for group in platform.get("groups", ["ungrouped"]):
                 instance_name = platform["name"]
-                connection_options = self.connection_options(instance_name)  # type: ignore[no-untyped-call]
+                connection_options = self.connection_options(instance_name)
                 molecule_vars = {
                     "molecule_file": "{{ lookup('env', 'MOLECULE_FILE') }}",
-                    "molecule_ephemeral_directory": "{{ lookup('env', 'MOLECULE_EPHEMERAL_DIRECTORY') }}",  # noqa: E501
-                    "molecule_scenario_directory": "{{ lookup('env', 'MOLECULE_SCENARIO_DIRECTORY') }}",  # noqa: E501
+                    "molecule_ephemeral_directory": "{{ lookup('env', 'MOLECULE_EPHEMERAL_DIRECTORY') }}",
+                    "molecule_scenario_directory": "{{ lookup('env', 'MOLECULE_SCENARIO_DIRECTORY') }}",
                     "molecule_yml": "{{ lookup('file', molecule_file) | from_yaml }}",
                     "molecule_instance_config": "{{ lookup('env', 'MOLECULE_INSTANCE_CONFIG') }}",
                     "molecule_no_log": "{{ lookup('env', 'MOLECULE_NO_LOG') or not "
@@ -654,44 +715,75 @@ class Ansible(base.Base):
                 for child_group in platform.get("children", []):
                     dd[group]["children"][child_group]["hosts"][instance_name] = connection_options
 
-        return self._default_to_regular(dd)  # type: ignore[no-untyped-call]
+        return self._default_to_regular(dd)
 
     @property
-    def inventory_directory(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def inventory_directory(self) -> str:
+        """Inventory directory path.
+
+        Returns:
+            Path to the directory containing inventory files.
+        """
         return self._config.scenario.inventory_directory
 
     @property
-    def inventory_file(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
-        return os.path.join(self.inventory_directory, "ansible_inventory.yml")  # noqa: PTH118
+    def inventory_file(self) -> str:
+        """Inventory file path.
+
+        Returns:
+            Path to ansible_inventory.yml
+        """
+        return str(Path(self.inventory_directory, "ansible_inventory.yml"))
 
     @property
-    def config_file(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
-        return os.path.join(  # noqa: PTH118
-            self._config.scenario.ephemeral_directory,
-            "ansible.cfg",
+    def config_file(self) -> str:
+        """Configuration file path.
+
+        Returns:
+            Path to ansible.cfg.
+        """
+        return str(
+            Path(
+                self._config.scenario.ephemeral_directory,
+                "ansible.cfg",
+            ),
         )
 
     @cached_property
-    def playbooks(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
+    def playbooks(self) -> ansible_playbooks.AnsiblePlaybooks:
+        """Ansible playbooks provisioner instance.
+
+        Returns:
+            AnsiblePlaybooks instance based on this config.
+        """
         return ansible_playbooks.AnsiblePlaybooks(self._config)
 
     @property
-    def directory(self):  # type: ignore[no-untyped-def]  # noqa: ANN201, D102
-        return os.path.join(  # noqa: PTH118
-            os.path.dirname(__file__),  # noqa: PTH120
-            os.path.pardir,
-            os.path.pardir,
-            "molecule",
-            "provisioner",
-            "ansible",
-        )
+    def directory(self) -> str:
+        """Ansible provisioner directory.
 
-    def cleanup(self):  # type: ignore[no-untyped-def]  # noqa: ANN201
+        Returns:
+            Path to the ansible provisioner directory.
+        """
+        return str(Path(__file__).parent.parent.parent / "molecule" / "provisioner" / "ansible")
+
+    def cleanup(self) -> None:
         """Execute `ansible-playbook` against the cleanup playbook and returns None."""
-        pb = self._get_ansible_playbook(self.playbooks.cleanup)  # type: ignore[no-untyped-call]
+        pb = self._get_ansible_playbook(self.playbooks.cleanup)
         pb.execute()
 
-    def connection_options(self, instance_name):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN201, D102
+    def connection_options(
+        self,
+        instance_name: str,
+    ) -> dict[str, Any]:
+        """Computed connection options combining instance and provisioner options.
+
+        Args:
+            instance_name: The instance to base the connection options on.
+
+        Returns:
+            The combined connection options.
+        """
         d = self._config.driver.ansible_connection_options(instance_name)
 
         return util.merge_dicts(
@@ -699,94 +791,112 @@ class Ansible(base.Base):
             self._config.config["provisioner"]["connection_options"],
         )
 
-    def check(self):  # type: ignore[no-untyped-def]  # noqa: ANN201
+    def check(self) -> None:
         """Execute ``ansible-playbook`` against the converge playbook with the ``--check`` flag."""
-        pb = self._get_ansible_playbook(self.playbooks.converge)  # type: ignore[no-untyped-call]
-        pb.add_cli_arg("check", True)  # noqa: FBT003
+        pb = self._get_ansible_playbook(self.playbooks.converge)
+        pb.add_cli_arg("check", value=True)
         pb.execute()
 
-    def converge(self, playbook=None, **kwargs):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN003, ANN201
+    def converge(self, playbook: str = "", **kwargs: object) -> str:
         """Execute ``ansible-playbook`` against the converge playbook. unless specified otherwise.
 
         Args:
             playbook: An optional string containing an absolute path to a playbook.
-            kwargs: An optional keyword arguments.
+            **kwargs: An optional keyword arguments.
 
         Returns:
             str: The output from the ``ansible-playbook`` command.
         """
-        pb = self._get_ansible_playbook(playbook or self.playbooks.converge, **kwargs)  # type: ignore[no-untyped-call]
+        pb = self._get_ansible_playbook(playbook or self.playbooks.converge, **kwargs)  # type: ignore[arg-type]
 
         return pb.execute()
 
-    def destroy(self):  # type: ignore[no-untyped-def]  # noqa: ANN201
+    def destroy(self) -> None:
         """Execute ``ansible-playbook`` against the destroy playbook and returns None."""
-        pb = self._get_ansible_playbook(self.playbooks.destroy)  # type: ignore[no-untyped-call]
+        pb = self._get_ansible_playbook(self.playbooks.destroy)
         pb.execute()
 
-    def side_effect(self, action_args=None):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN201
-        """Execute ``ansible-playbook`` against the side_effect playbook and returns None."""
+    def side_effect(self, action_args: list[str] | None = None) -> None:
+        """Execute ``ansible-playbook`` against the side_effect playbook.
+
+        Args:
+            action_args: Arguments to pass to the side_effect playbook.
+        """
+        playbooks = []
         if action_args:
             playbooks = [
-                self._get_ansible_playbook(self._config.provisioner.abs_path(playbook))  # type: ignore[no-untyped-call]
-                for playbook in action_args
+                self._get_ansible_playbook(self.abs_path(playbook)) for playbook in action_args
             ]
         else:
-            playbooks = [self._get_ansible_playbook(self.playbooks.side_effect)]  # type: ignore[no-untyped-call]
+            playbooks = [self._get_ansible_playbook(self.playbooks.side_effect)]
         for pb in playbooks:
             pb.execute()
 
-    def create(self):  # type: ignore[no-untyped-def]  # noqa: ANN201
+    def create(self) -> None:
         """Execute ``ansible-playbook`` against the create playbook and returns None."""
-        pb = self._get_ansible_playbook(self.playbooks.create)  # type: ignore[no-untyped-call]
+        pb = self._get_ansible_playbook(self.playbooks.create)
         pb.execute()
 
-    def prepare(self):  # type: ignore[no-untyped-def]  # noqa: ANN201
+    def prepare(self) -> None:
         """Execute ``ansible-playbook`` against the prepare playbook and returns None."""
-        pb = self._get_ansible_playbook(self.playbooks.prepare)  # type: ignore[no-untyped-call]
+        pb = self._get_ansible_playbook(self.playbooks.prepare)
         pb.execute()
 
-    def syntax(self):  # type: ignore[no-untyped-def]  # noqa: ANN201
+    def syntax(self) -> None:
         """Execute `ansible-playbook` against the converge playbook with the -syntax-check flag."""
-        pb = self._get_ansible_playbook(self.playbooks.converge)  # type: ignore[no-untyped-call]
-        pb.add_cli_arg("syntax-check", True)  # noqa: FBT003
+        pb = self._get_ansible_playbook(self.playbooks.converge)
+        pb.add_cli_arg("syntax-check", value=True)
         pb.execute()
 
-    def verify(self, action_args=None):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN201
-        """Execute ``ansible-playbook`` against the verify playbook and returns None."""
+    def verify(self, action_args: list[str] | None = None) -> None:
+        """Execute ``ansible-playbook`` against the verify playbook.
+
+        Args:
+            action_args: Arguments to pass on to the verify playbook.
+        """
+        playbooks = []
         if action_args:
-            playbooks = [self._config.provisioner.abs_path(playbook) for playbook in action_args]
-        else:
+            playbooks = [self.abs_path(playbook) for playbook in action_args]
+        elif self.playbooks.verify:
             playbooks = [self.playbooks.verify]
         if not playbooks:
             LOG.warning("Skipping, verify playbook not configured.")
             return
         for playbook in playbooks:
             # Get ansible playbooks for `verify` instead of `provision`
-            pb = self._get_ansible_playbook(playbook, verify=True)  # type: ignore[no-untyped-call]
+            pb = self._get_ansible_playbook(playbook, verify=True)
             pb.execute()
 
-    def write_config(self):  # type: ignore[no-untyped-def]  # noqa: ANN201
+    def write_config(self) -> None:
         """Write the provisioner's config file to disk and returns None."""
-        template = util.render_template(  # type: ignore[no-untyped-call]
-            self._get_config_template(),  # type: ignore[no-untyped-call]
+        template = util.render_template(
+            self._get_config_template(),
             config_options=self.config_options,
         )
         util.write_file(self.config_file, template)
 
-    def manage_inventory(self):  # type: ignore[no-untyped-def]  # noqa: ANN201
+    def manage_inventory(self) -> None:
         """Manage inventory for Ansible and returns None."""
-        self._write_inventory()  # type: ignore[no-untyped-call]
-        self._remove_vars()  # type: ignore[no-untyped-call]
+        self._write_inventory()
+        self._remove_vars()
         if not self.links:
-            self._add_or_update_vars()  # type: ignore[no-untyped-call]
+            self._add_or_update_vars()
         else:
-            self._link_or_update_vars()  # type: ignore[no-untyped-call]
+            self._link_or_update_vars()
 
-    def abs_path(self, path: str) -> str | None:  # noqa: D102
-        return util.abs_path(os.path.join(self._config.scenario.directory, path))  # noqa: PTH118
+    def abs_path(self, path: str | Path) -> str:
+        """Return absolute scenario-adjacent path.
 
-    def _add_or_update_vars(self):  # type: ignore[no-untyped-def]  # noqa: ANN202
+        Args:
+            path: Scenario-adjacent relative path.
+
+        Returns:
+            Absolute path.
+        """
+        path = Path(self._config.scenario.directory) / path
+        return str(util.abs_path(path))
+
+    def _add_or_update_vars(self) -> None:
         """Create host and/or group vars and returns None."""
         # Create the hosts extra inventory source (only if not empty)
         hosts_file = os.path.join(self.inventory_directory, "hosts")  # noqa: PTH118
@@ -804,26 +914,22 @@ class Ansible(base.Base):
                 vars_target = self.group_vars
 
             if vars_target:
-                target_vars_directory = os.path.join(  # noqa: PTH118
-                    self.inventory_directory,
-                    target,
-                )
-
-                if not os.path.isdir(util.abs_path(target_vars_directory)):  # type: ignore[arg-type]  # noqa: PTH112
-                    os.mkdir(util.abs_path(target_vars_directory))  # type: ignore[arg-type]  # noqa: PTH102
+                target_vars_directory = util.abs_path(Path(self.inventory_directory) / target)
+                if not target_vars_directory.is_dir():
+                    target_vars_directory.mkdir()
 
                 for target in vars_target:  # noqa: PLW2901
                     target_var_content = vars_target[target]
-                    path = os.path.join(util.abs_path(target_vars_directory), target)  # type: ignore[arg-type]  # noqa: PTH118
+                    path = target_vars_directory / target
                     util.write_file(path, util.safe_dump(target_var_content))
 
-    def _write_inventory(self):  # type: ignore[no-untyped-def]  # noqa: ANN202
+    def _write_inventory(self) -> None:
         """Write the provisioner's inventory file to disk and returns None."""
-        self._verify_inventory()  # type: ignore[no-untyped-call]
+        self._verify_inventory()
 
         util.write_file(self.inventory_file, util.safe_dump(self.inventory))
 
-    def _remove_vars(self):  # type: ignore[no-untyped-def]  # noqa: ANN202
+    def _remove_vars(self) -> None:
         """Remove hosts/host_vars/group_vars and returns None."""
         for name in ("hosts", "group_vars", "host_vars"):
             d = os.path.join(self.inventory_directory, name)  # noqa: PTH118
@@ -832,7 +938,7 @@ class Ansible(base.Base):
             elif os.path.isdir(d):  # noqa: PTH112
                 shutil.rmtree(d)
 
-    def _link_or_update_vars(self):  # type: ignore[no-untyped-def]  # noqa: ANN202
+    def _link_or_update_vars(self) -> None:
         """Create or updates the symlink to group_vars and returns None."""
         for d, source in self.links.items():
             target = os.path.join(self.inventory_directory, d)  # noqa: PTH118
@@ -853,29 +959,37 @@ class Ansible(base.Base):
             LOG.debug(msg)
             os.symlink(source, target)
 
-    def _get_ansible_playbook(self, playbook, verify=False, **kwargs):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN003, ANN202, FBT002
+    def _get_ansible_playbook(
+        self,
+        playbook: str | None,
+        verify: bool = False,  # noqa: FBT001, FBT002
+        **kwargs: object,
+    ) -> ansible_playbook.AnsiblePlaybook:
         """Get an instance of AnsiblePlaybook and returns it.
 
         Args:
             playbook: A string containing an absolute path to a provisioner's playbook.
             verify: An optional bool to toggle the Playbook mode between provision and verify.
                 False: provision; True: verify. Default is False.
-            kwargs: An optional keyword arguments.
+            **kwargs: An optional keyword arguments.
+
+        Returns:
+            An AnsiblePlaybook object.
         """
         return ansible_playbook.AnsiblePlaybook(
             playbook,
             self._config,
-            verify,
+            verify=verify,
             **kwargs,
         )
 
-    def _verify_inventory(self):  # type: ignore[no-untyped-def]  # noqa: ANN202
+    def _verify_inventory(self) -> None:
         """Verify the inventory is valid and returns None."""
         if not self.inventory:
             msg = "Instances missing from the 'platform' section of molecule.yml."
             util.sysexit_with_message(msg)
 
-    def _get_config_template(self):  # type: ignore[no-untyped-def]  # noqa: ANN202
+    def _get_config_template(self) -> str:
         """Return a config template string.
 
         Returns:
@@ -890,17 +1004,20 @@ class Ansible(base.Base):
 {% endfor -%}
 """.strip()
 
-    def _vivify(self):  # type: ignore[no-untyped-def]  # noqa: ANN202
+    def _vivify(self) -> Vivify:
         """Return an autovivification default dict.
 
-        Return:
-            dict
+        Returns:
+            A defaultdict whose default value is other defaultdict objects (and so on).
         """
         return collections.defaultdict(self._vivify)
 
-    def _default_to_regular(self, d):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN202
+    def _default_to_regular(
+        self,
+        d: dict[str, Any] | collections.defaultdict[str, Any],
+    ) -> dict[str, Any]:
         if isinstance(d, collections.defaultdict):
-            d = {k: self._default_to_regular(v) for k, v in d.items()}  # type: ignore[no-untyped-call]
+            d = {k: self._default_to_regular(v) for k, v in d.items()}
 
         return d
 
@@ -911,6 +1028,9 @@ class Ansible(base.Base):
         """Return list of ansible module includes directories.
 
         Adds modules directory from molecule and its plugins.
+
+        Returns:
+            List of module includes directories.
         """
         paths: list[str | None] = []
         if os.environ.get("ANSIBLE_LIBRARY"):
@@ -920,7 +1040,7 @@ class Ansible(base.Base):
             util.abs_path(os.path.join(self._get_plugin_directory(), "modules")),  # noqa: PTH118
         )
 
-        for d in drivers():
+        for d in drivers().values():
             p = d.modules_dir()
             if p:
                 paths.append(p)
@@ -949,11 +1069,15 @@ class Ansible(base.Base):
 
         return [path for path in paths if path is not None]
 
-    def _get_filter_plugin_directory(self):  # type: ignore[no-untyped-def]  # noqa: ANN202
+    def _get_filter_plugin_directory(self) -> str:
         return util.abs_path(os.path.join(self._get_plugin_directory(), "filter"))  # noqa: PTH118
 
     def _get_filter_plugins_directories(self) -> list[str]:
-        """Return list of ansible filter plugins includes directories."""
+        """Return list of ansible filter plugins includes directories.
+
+        Returns:
+            List of filter includes directories.
+        """
         paths: list[str | None] = []
         if os.environ.get("ANSIBLE_FILTER_PLUGINS"):
             paths = list(
@@ -962,7 +1086,7 @@ class Ansible(base.Base):
 
         paths.extend(
             [
-                self._get_filter_plugin_directory(),  # type: ignore[no-untyped-call]
+                self._get_filter_plugin_directory(),
                 util.abs_path(
                     os.path.join(  # noqa: PTH118
                         self._config.scenario.ephemeral_directory,
@@ -991,5 +1115,5 @@ class Ansible(base.Base):
 
         return [path for path in paths if path is not None]
 
-    def _absolute_path_for(self, env, key):  # type: ignore[no-untyped-def]  # noqa: ANN001, ANN202
-        return ":".join([self.abs_path(p) for p in env[key].split(":")])  # type: ignore[misc]
+    def _absolute_path_for(self, env: dict[str, str], key: str) -> str:
+        return ":".join([self.abs_path(p) for p in env[key].split(":")])
